part of geopointer;

/// Circle-base GEO algorithms.
///
/// Circle uses by default the Vincenty-Algorithm for distance computations
class GCircle {
  final double radius;
  final GeoLatLng center;
  final DistanceCalculator _calculator;

  GCircle(final GeoLatLng this.center, this.radius,
      {final DistanceCalculator calculator: const Vincenty()})
      : _calculator = calculator;

  /// Checks if a [point] is inside the given [GCircle]
  ///
  ///     final Circle circle = new Circle(new LatLng(0.0,0.0), 111319.0);
  ///     final LatLng newPos = new LatLng(1.0,0.0);
  ///
  ///     expect(circle.isPointInside(newPos),isTrue);
  ///
  ///     final Circle circle2 = new Circle(new LatLng(0.0,0.0), 111318.0);
  ///
  ///     expect(circle2.isPointInside(newPos),isFalse);
  ///
  bool isPointInside(final GeoLatLng point) {
    GValidate.notNull(point);

    final GDistance distance = new GDistance(calculator: _calculator);

    final double dist = distance(center, point).toDouble();
    return dist <= radius;
  }

//- private -----------------------------------------------------------------------------------
}
